<?php

/* general_2.php
*
    Copyright (C) 2002-2005  Hastymail Development group

    This file is part of Hastymail.

    Hastymail is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Hastymail is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Hastymail; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

* $Id $
*/

/**
 *
 *  Spell check given text
 *
 *  @param      $text       string      text to spell check
 *  @param      $dict       string      selected dictionary     
 *  @return                 array       suggested spellings array
 */
function hm_spellcheck($text, $dict, $subject=false) {
    $new_text = '';
    $pspell_link = @pspell_new(trim($dict));
    $text = str_replace("\n", " \n", $text);
    $words =  preg_split ("/[ ]+/", $text);
    if ($subject) {
        $prefix = 'sub_spell_';
    }
    else {
        $prefix = 'spell_';
    }
    if (!$pspell_link) {
        $_SESSION['notices'][] = $_SESSION['str'][381].' '.$dict;
        return $text;
    }
    foreach ($words as $index => $word) {
        $check_word = str_replace(array(':', ',', ';', '_','.', '"', "'", '(', ')',
                      '?', '!', '%', '$', '*', '/', '>', '<'), '',
                      trim($word));
        if (strlen($check_word) > 1 && !is_numeric($check_word) &&
            $check_word{0} != '-' && strpos($word, '@') === false) {
            if (!pspell_check($pspell_link, $check_word)) {
                $suggests = pspell_suggest($pspell_link, $check_word);
                $options = '';
                foreach($suggests as $s) {
                    $options .= '<option value="'.$s.'">'.$s.'</option>';
                }
                $new_text .= str_replace($check_word, '<select name="'.$prefix.$index.
                             '"><option value="'.$check_word.'">'.$check_word.'</option>'.
                             '<option disabled value="">'.
                            str_repeat('-', strlen($check_word)).'</option>'.
                             $options.'</select>', hm_htmlize($word));
            }
            else {
                $new_text .= hm_htmlize($word);
            }
        }
        else {
            $new_text .= hm_htmlize($word);
        }
        $new_text .= ' ';
    }
    return $new_text;
}

/**
 *
 *  Apply spellcheck changes to message body
 *
 *  @param      $text       string      text to spell check
 *  @param      $post       array       POST vars 
 *  @return                 string      modified message text
 */
function hm_apply_spellcheck($text, $post, $subject=false) {
    $new_text = str_replace("\n", " \n", $text);
    $words =  preg_split ("/[ ]+/", $new_text);
    if ($subject) {
        $prefix = 'sub_spell_';
    }
    else {
        $prefix = 'spell_';
    }
    foreach ($words as $index => $word) {
        $check_word = str_replace(array(':', ',', ';', '_','.', '"', "'", '(', ')',
                      '?', '!', '%', '$', '*', '/', '>', '<'), '',
                      trim($word));
        if (strlen($check_word) > 1 && !is_numeric($check_word) &&
            $check_word{0} != '-' && strpos($word, '@') === false) {
            if (isset($post[$prefix.$index])) {
                $pos = strpos($text, $check_word);
                $text = substr_replace($text, $post[$prefix.$index], $pos,
                strlen($check_word));

            }
        }
    }
    return $text;
}

/**
 *
 * Parse an address field into valid email addresses
 *
 *  @param  $string     string      address field string
 *  @param  $get_name   boolean     if true get names for each address as well
 *  @return             array       actual email addresses
 */
function hm_parse_address ($string, $get_names=false) {
    $parts = array();
    $addy = array();
    $string = str_replace(',', ', ', $string);
    $string = stripslashes($string);
    $string = str_replace(array('&lt;', '&gt;'), array('<', '>'), $string);

    /* get rid of any comments in parens, even if nested */
    if (strpos($string, '(') !== false) {
        $max = strlen($string);
        $line = '';
        $level = 0;
        for ($i=0;$i<$max;$i++) {
            if ($string{$i} == '(') {
                $level++;
            }
            if ($level == 0) {
                $line .= $string{$i};
            }
            if ($string{$i} == ')') {
                $level--;
            }
        }
        /* if we have mis-matched parens parse the whole bugger anyway */
        if (!$line) {
            $line = $string;
        }
    }
    else {
        $line = $string;
    }
    /* break up the line, getting rid of any quoted stuff */
    $line = trim($line);
    $line = str_replace(';', ',', $line);
    $line = str_replace(',', ' , ', $line);
    $level = 0;
    while ($line) {
        $chunk = false;
        switch ($line{0}) {
            case ',':
                $level++;
                $line = substr($line, 1);
                break;
            case '"':
                if (preg_match("/^(\"[^\"\\\]*(?:\\\.[^\"\\\]*)*\")/",
                    $line, $matches)) {
                    $chunk = $matches[1];
                }
                $line = substr($line, strlen($chunk));
                $chunk = false;
                break;
            case ' ':
                $line = substr($line, 1);
                break;
            default:
                if (strpos($line, ' ') !== false) {
                    $chunk = substr($line, 0, strpos($line, ' '));
                    $line = substr($line, strlen($chunk));
                }
                else {
                    $chunk = $line;
                    $line = false;
                }
                break;
        }
        if (is_string($chunk)) {
            $parts[$level][] = trim($chunk);
        }
    }
    /* loop through the parts and get valid addys */
    foreach ($parts as $array) {
        if (count($array) == 1) {
            $addy[] = preg_replace("/((^&lt;|^<)|(&gt$|>$))/", '',
                      rtrim($array[0], ',;'));
        }
        elseif (count($array) > 1) {
            foreach ($array as $part) {
                if (strpos($part, '@') !== false) {
                    $addy[] = preg_replace("/((^&lt;|^<)|(&gt$|>$))/", '',
                        rtrim($part, ',;'));
                break;
                }
            }
        }
    }
    if (empty($addy) && $string != '') {
        if (strpos($string, ',')) {
            $temp_addy = explode(',', $string);
            foreach ($temp_addy as $temp) {
                if (!preg_match("/(\(|\)|\"|\')/", $temp)) {
                    $addy[] = trim($temp);
                }
            }
        }
        else {
            $addy[] = $string;
        }
    }
    return $addy;
}

/**
 *  find and return parts of a message
 *
 *  @param      $imap_stream    resource    connection to imap
 *  @param      $structure      array       message mime structure
 *  @param      $uid            int         message uid
 *  @param      $part           float       part to return
 *  @param      $reply          boolean     format as a reply
 *  @param      $clean          boolean     don't add line break char (\)
 *  @param      $cutoff         boolean     flag to break LARGE messages
 *  @returns                    string      formatted message part
 *
 */
function &hm_find_display_part ($imap_stream, $structure, $uid, $part=false, 
                               $reply=false, $clean=false, $cutoff=false) {
    $str = $_SESSION['str'];
    $result_fallback = false;
    if (!$part) {
        /* here we don't know the text part we want so find the first one */
        if (is_array($structure)) {
            foreach($structure as $index => $array) {
                if ($array['type'].$array['subtype'] == 'textplain' ||
                    $array['type'].$array['subtype'] == 'textenriched') {
                    $text_part = &hm_get_message_part($imap_stream, $uid, $index,
                                 $cutoff);
                    if (isset($array['charset'])) {
                        $chr = $array['charset'];
                    }
                    else {
                        $chr = false;
                    }
                    if ($array['encoding'] == 'quoted-printable') {
                        $text_part = &hm_quoted_decode($text_part);
                    }
                    elseif ($array['encoding'] == 'base64') {
                        $text_part = base64_decode($text_part);
                    }
                    $text_part = &hm_text_cleanup($text_part, $reply, $clean);
                    if ($array['type'].$array['subtype'] == 'textenriched') {
                        if ($reply) {
                            $text_part = str_replace("\n>\n>", "\n>", $text_part);
                        }
                        else {
                            $text_part = str_replace("\n\n", "\n", $text_part);
                        }
                    }
                    if (trim($text_part) != '') {
                        $result = array($index, $text_part, array($array['encoding'], 
                                        'text', 'plain', 'textplain', $chr));
                        return $result;
                    }
                    else {
                        $result_fallback = array($index, $text_part,
                                           array($array['encoding'], 'text', 'plain',
                                           'textplain', $chr));
                    }
                }
                if (isset($array['subs'])) {
                    $result = &hm_find_display_part ($imap_stream, $array['subs'], $uid, 
                                                    false, $reply, $clean, $cutoff);
                    return $result;
                }
            }
        }
    }
    else {
        /* this is for parts selected from the message parts links in message.php */
        $dsp_type = false;
        $types = array(
        'text:plain'  => array ('textplain', 'messagedisposition-notification',
                                'messagedelivery-status', 'textenriched', 
                                'textrfc822-headers', 'messagerfc822-headers',
                                'applicationpgp-signature'),
        'image:jpeg'  => array('imagejpeg', 'imagepjpeg', 'imagejpg'),
        'image:png'   => array('imagepng'),
        'image:gif'   => array('imagegif'),
        'text:html'   => array('texthtml'));

        $attributes = &hm_in_multi_array ($part, $structure);
        if (isset($attributes[1]['type'])) {
            $type = $attributes[1]['type'];
        }
        else {
            $type = false;
        }
        if (isset($attributes[1]['name'])) {
            $name = $attributes[1]['name'];
        }
        else {
            $name = false;
        }
        if (isset($attributes[1]['subtype'])) {
            $subtype = $attributes[1]['subtype'];
        }
        else {
            $subtype = false;
        }
        if (isset($attributes[1]['encoding'])) {
            $decode = $attributes[1]['encoding'];
        }
        else {
            $decode = false;
        }
        if (isset($attributes[1]['charset'])) {
           $chr = $attributes[1]['charset'];
        }
        else {
            $chr = false;
        }
        $dsp_type = $type.$subtype;
        $dsp_flag = false;
        foreach ($types as $key => $array) {
            if (in_array($dsp_type, $array)) {
                $dsp_type = str_replace(':', '', $key);
                $dsp_flag = true;
                break;
            }
        }
        if (!$dsp_flag) {
            if ($type == 'text') {
                $dsp_type = 'textplain';
            }
            if ($type == 'image') {
                $dsp_type = 'image'.$subtype;
            }
        }
        $message_part = &hm_get_message_part($imap_stream, $uid, $part, $cutoff);
        if ($decode == 'quoted-printable') {
            $message_part = &hm_quoted_decode($message_part); 
        }
        elseif ($decode == 'base64') {
            $message_part = base64_decode($message_part);
        }
        if ($dsp_type == 'textplain') {
            $message_part = str_replace("\0", '', $message_part);
            if (!$clean) {
                $message_part = &hm_text_cleanup($message_part);
            }
            if ($type.$subtype == 'textenriched') {
                $message_part = str_replace("\n\n", "\n", $message_part);
            }
        }
        $result = array($part, $message_part, 
                        array( $decode, $type, $subtype, $dsp_type, $chr, $name));
        return $result;
    }
    /* if we could not find a text part return a message */
    if (!$result_fallback) {
        $text_part = "\n\n     ".$str[194]."\n\n";
        $result = array('0', $text_part, array('none', 'text', 'plain', 'textplain'));
        return $result;
    }
    else {
        return $result_fallback;
    }
} 

/**
 *  special strtotime that adjusts for timezone setting
 *
 *  @param      $string         string      input date string
 *  @returns                    string      adjusted time
 *
 */
function hm_strtotime ($string) {
    $timestamp = @strtotime($string);
    $current_timezone = date('Z');
    $current_daylight = date('I');
    if (!isset($_SESSION['settings']['timezone']) && 
        !isset($_SESSION['settings']['daylight_savings'])) {
        return $timestamp;
    }
    elseif (($current_daylight && $_SESSION['settings']['timezone'] ==
           ($current_timezone - 3600)) || (!$current_daylight &&
            $_SESSION['settings']['timezone'] == $current_timezone)) {
        return $timestamp;
    }
    else {
        if (!isset($_SESSION['settings']['timezone'])) {
            $timezone = $current_timezone;
        }
        else {
            $timezone = $_SESSION['settings']['timezone'];
        }
        if (!isset($_SESSION['settings']['daylight_savings'])) {
            $daylight = $current_daylight;
        }
        else {
            $daylight = $_SESSION['settings']['daylight_savings'];
        }
    }
    $gmt = $timestamp - $current_timezone;
    $adjusted = $gmt + $timezone;
    if ($daylight) {
        $adjusted += 3600;
    }
    return $adjusted;
}

function hm_build_timezones() {
    $tzs = array();
    $min = 1;
    $hour = 0;
    for ($i=0;$i<=43200;$i+=900) {
        switch ($min) {
            case 1:
                $label = $hour.':00';
                break;
            case 2:
                $label = $hour.':15';
                break;
            case 3:
                $label = $hour.':30';
                break;
            case 4:
                $label = $hour.':45';
                break;
        }
        if ($i == 0) {
            $tzs[0] = '00:00';
        }
        else {
            if (strlen($label) == 4) {
                $label = '0'.$label;
            }
            $tzs['-'.$i] = '-'.$label;
            $tzs['+'.$i] = '+'.$label;
        }
        if ($min == 4) {
            $min = 1;
            $hour++;
        }
        else {
            $min++;
        }
    }
    krsort($tzs);
    return $tzs;
}
?>
